@@ -431,6 +431,8 @@ def char_at(x,y,char,color=[0,0]):
     code = color[0]*16+color[1]
     ansi = "\u001b[38;5;"+str(code)+"m"
     if(y>=0 and x>=0 and x<len(output[0]) and y<len(output)):
+        x= round(x)
+        y= round(y)
         output[y][x][1] = char
         output[y][x][0] = ansi
 
@@ -826,7 +828,67 @@ def import_obj(filename):
 #----------------------------------------------------------------------#
 #----------------------------------------------------------------------#
 #----------------------------------------------------------------------#
+
+
+
+#----------------------------------------------------------------------#
+#------------plotting graphs and reading csv---------------------------#
 #----------------------------------------------------------------------#
 #----------------------------------------------------------------------#
-#----------------------------------------------------------------------#
-#----------------------------------------------------------------------#
+
+class figure:
+    def __init__(self,x,y,width,height,xr,yr,k=5,l=5,m=4):
+        self.x = x
+        self.y = y
+        self.width=width
+        self.height=height
+        self.points = []
+        self.lines = []
+        self.xr=xr
+        self.yr=yr
+        self.px=[self.x+2,self.width-2]
+        self.py=[self.y+2,self.height-2]
+        self.plotrange(self.xr,self.yr,k,l,m)
+        self.__drawaxis()
+    def __drawaxis(self):
+        line_2d(self.x,self.y+self.height-2,self.width+self.x,self.y+self.height-2,[15,15],52,False)
+        line_2d(self.x+2,self.y+2,self.x+2,self.height+self.y,[15,15],42,False)
+        point(self.x+2,self.y+self.height-2,[15,15],1)
+    def __maprange(self,map,a=[10,100],b=[200,20]):
+        k,l,m,n=min(a),max(a),min(b),max(b)
+        out = (((map-k)/l)*n)+m
+        return out
+    def plotrange(self,x,y,k=5,l=5,m=4):
+        px=[self.x+2,self.width-2]
+        for i in range(0,max(x)-min(x),k):
+            xk = self.__maprange(i,x,self.px)
+            text(xk,self.y+self.height+1,str(i),[15,14],False)
+        py=[self.y+2,self.height-2]
+        for i in range(0,max(y)-min(y),l):
+            yk = self.__maprange(i,y,self.py)
+            text(self.x+1-m,2*self.y+self.height-yk,str(i),[15,14],False)
+    def point(self,p):
+        for i in p:
+            xk = self.__maprange(i[0],self.xr,self.px)
+            yk = self.__maprange(i[1],self.yr,self.py)
+            char_at(xk, 2*self.y+self.height-yk,'*',[0,1])
+    def line(self,p,color=[5,5],intensity=0):
+        for i in p:
+            try:
+                prevx=xk
+                prevy=yk
+            except :
+                pass
+            xk = self.__maprange(i[0],self.xr,self.px)
+            yk = self.__maprange(i[1],self.yr,self.py)
+            yk = 2*self.y+self.height-yk
+            try:
+                line_2d(xk,yk,prevx,prevy,color,intensity,False)
+                char_at(xk,yk,'*',[0,1])
+                char_at(prevx,prevy,'*',[0,1])
+            except:
+                pass
+
+                
+
+
